require 'spec_helper'
require 'persisto/repository'

describe Persisto::Repository do
	let(:mapper) { stub }
	let(:adaptor){ stub }
	let(:repo_class){ Class.new{ include Persisto::Repository } }
	let(:repo){ repo_class.new(adaptor) }

	before { repo_class.set_mapper(mapper) }

	describe '::set_mapper' do
		it 'assigns mapper' do
			repo_class.set_mapper('blah')
			repo_class.mapper.must_equal 'blah'
		end
	end

	describe '::mapper' do
		context 'when mapper assigned' do
			before { repo_class.set_mapper(mapper) }

			it 'returns mapper' do
				repo_class.mapper.must_equal mapper
			end
		end

		context 'when mapper not assigned' do
			before { repo_class.set_mapper(nil) }

			it 'raises error' do
				proc{ repo_class.mapper }.must_raise Persisto::RepositoryError
			end
		end
	end

	describe '::register_store' do
		let(:store_class){stub}

		before { repo_class.register_store :mystore, store_class }

		it 'adds store to registry' do
			repo_class.store_registry.must_equal({:mystore => store_class})
		end

		it 'adds attr_reader for store' do
			repo.respond_to?(:mystore).must_equal true
		end
	end

	describe 'store method' do
		context 'when store initialize' do
			it 'returns store' do
				repo.instance_variable_set(:@stores, {:blah => 'blah'})
				repo.send(:store, :blah).must_equal 'blah'
			end
		end

		context 'when store not initialized' do
			let(:mystore_class){ stub }
			let(:store){ stub }
			before { repo_class.register_store :mystore, mystore_class }

			it 'initializes store' do
				mystore_class.expects(:new).with(adaptor, mapper).returns(store)
				repo.mystore
			end

			it 'returns store' do
				mystore_class.expects(:new).with(adaptor, mapper).returns(store)
				repo.mystore.must_equal store
			end
		end
	end
end