require 'spec_helper'
require 'persisto/store'

describe Persisto::Store do
	let(:mapper) { stub }
	let(:adaptor){ stub }
	let(:store_class){ Class.new{ include Persisto::Store } }
	let(:store){ store_class.new(adaptor, mapper) }

	describe '::table_name' do
		let(:table_name) {'blah'}

		before { store_class.set_table_name(table_name) }

		it 'returns table name' do
			store_class.table_name.must_equal table_name
		end
	end

	describe '::set_table_name' do
		it 'sets table name class var' do
			store_class.set_table_name('blah')
			store_class.table_name.must_equal('blah')
		end
	end

	describe '#all' do
		let(:query_object){stub}
		let(:data){stub}
		let(:entity){stub}

		it 'returns list of entities' do
			query_object.expects(:store=).with(store)
			adaptor.expects(:select).with(query_object).returns([data])
			mapper.expects(:parse).with(data).returns(entity)
			store.all(query_object).must_equal [entity]
		end
	end

	describe '#first' do
		let(:query_object){stub}
		let(:data){stub}
		let(:entity){stub}

		it 'returns first entity' do
			query_object.expects(:store=).with(store)
			adaptor.expects(:select).with(query_object).returns([data])
			mapper.expects(:parse).with(data).returns(entity)
			store.first(query_object).must_equal entity
		end
	end

	describe '#first!' do
		let(:query_object){stub}
		let(:data){stub}
		let(:entity){stub}

		context 'when records found in database' do
			it 'returns first entity' do
				query_object.expects(:store=).with(store)
				adaptor.expects(:select).with(query_object).returns([data])
				mapper.expects(:parse).with(data).returns(entity)
				store.first(query_object).must_equal entity
			end
		end

		context 'when records not found in database' do
			it 'returns first entity' do
				query_object.expects(:store=).with(store)
				adaptor.expects(:select).with(query_object).returns([])
				proc{ store.first!(query_object) }.must_raise(Persisto::EntityNotFound)
			end
		end
	end

	describe '#insert' do
		let(:query_object){stub}
		let(:primary_key){123}

		it 'inserts query into adaptor' do
			query_object.expects(:store=).with(store)
			adaptor.expects(:insert).with(query_object).returns(primary_key)
			store.insert(query_object).must_equal primary_key
		end
	end

	describe '#update' do
		let(:query_object){stub}
		let(:changed_rows){1}

		it 'updates query into adaptor' do
			query_object.expects(:store=).with(store)
			adaptor.expects(:update).with(query_object).returns(changed_rows)
			store.update(query_object).must_equal changed_rows
		end
	end

	describe '#delete' do
		let(:query_object){stub}
		let(:changed_rows){1}

		it 'deletes query into adaptor' do
			query_object.expects(:store=).with(store)
			adaptor.expects(:delete).with(query_object).returns(changed_rows)
			store.delete(query_object).must_equal changed_rows
		end
	end

	describe '#transaction' do
		it 'yields to adaptor transaction method' do
			adaptor.expects(:transaction).yields
			store.transaction
		end
	end
end